﻿

using System;
using System.Linq;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace SPQR {
	public class VDDollworks{
		VirtualDoll X;
		bool processReset;
		bool isStatic;
		Atom sourceAtom_p67;
		Vector3 positionNeutral_p67;
		Vector3[] trackedLog = new Vector3[5];
		Rigidbody destinationRB;

		FreeControllerV3 headController;
		FreeControllerV3 neckController;
		FreeControllerV3 chestController;
		FreeControllerV3 pelvisController;
		FreeControllerV3 hipController;

		LineRenderer dollline;


		public List<string> modes = new List<string>(new string[] { "Ride", "BJ", "Thrust"});
		public List<string> modesSticky = new List<string>(new string[] { "None", "VR Left Hand", "VR Right Hand", "Tracker1", "Tracker2", "Tracker3", "Tracker4", "Tracker5", "Tracker6", "Tracker7", "Tracker8" });


		Dictionary<string, bool> initiated = new Dictionary<string, bool>() {
			{ "VirtualDollControl", false }
		};

		/* CODE*/


		public VDDollworks(VirtualDoll script)
		{
			X = script;
			
		}
		public void Init_p67() {

			PrepareLines_p67();
			if (!X.enableFeatures_p67["enable_virtualdoll"].val) return;
			headController = X.PersonAtom.GetStorableByID("headControl") as FreeControllerV3;
			neckController = X.PersonAtom.GetStorableByID("neckControl") as FreeControllerV3;
			chestController = X.PersonAtom.GetStorableByID("chestControl") as FreeControllerV3;
			pelvisController = X.PersonAtom.GetStorableByID("pelvisControl") as FreeControllerV3;
			hipController = X.PersonAtom.GetStorableByID("hipControl") as FreeControllerV3;

			chestController.transform.localEulerAngles = Vector3.zero;
			hipController.transform.localEulerAngles = Vector3.zero;
			SetTrackedController_p67();
			
		}

		public void PrepareLines_p67()
        {
			var go1 = GameObject.Find("SPQRlineVDline");
			if (go1 == null) go1 = new GameObject("SPQRlineVDline");

			dollline = go1.GetComponent<LineRenderer>();
			if (dollline == null) dollline = go1.AddComponent<LineRenderer>();

			var m = new Material(Shader.Find("Sprites/Default"));
			dollline.material = m;

			dollline.widthMultiplier = 0.0025f;

			dollline.sortingOrder = 1;

			Color c1 = new Color(0.3f, 1f, 0.3f, 0.7f);
			Color c2 = new Color(1f, 0.3f, 0.3f, 0.7f);
			dollline.SetColors(c1, c2);


		}
		public void CheckAtomExists()
        {
			if (!X.enableFeatures_p67["enable_virtualdoll"].val) return;
			if (GetTrackedController_p67() != null) return;
			initiated["VirtualDollControl"] = false;
			X.StartCoroutine(X.CoroutineCreateAtom_p67("Empty", X.AtomUidAddSuffix_p67("VirtualDollControl")));
		}

		public void EnableVirtualdoll(bool b)
        {
			if (b) {
				Init_p67();
			}
			else
            {
				var a = GetTrackedController_p67();

				if (a != null)
                {
					a.Remove();
                }
            }
        }



	
		public void Update_p67() {
			PositionLines_p67();
		}


		
		public void FixedUpdate_p67() {
			if (!X.enableFeatures_p67["enable_virtualdoll"].val) return;
			ProcessTrackedPosition_p67();
			UpdatePosition_p67();
			CheckReset_p67();
		}

		public void UpdateSecond_p67()
		{
			if (!X.enableFeatures_p67["enable_virtualdoll"].val) return;
            if (!initiated["VirtualDollControl"])
            {
				if(GetTrackedController_p67()!= null) InitiateAliveVirtualDollControl();
            }
			CheckAtomExists();

			ProcessTrackedReset_p67();
		}

		public void UpdateConstant_p67()
		{
			if (!X.enableFeatures_p67["enable_virtualdoll"].val) return;
			UpdatePosition_p67();
			CheckReset_p67();
		}
		public void UpdateConstant5_p67()
		{
			//SuperController.LogMessage("s: " + positionNeutral_p67.ToString());
		}

		public void PositionLines_p67()
		{
			bool ok_p67 = true;

			if (!GetTrackedController_p67()) ok_p67 = false;
			if (!X.enableFeatures_p67["enable_virtualdoll"].val) ok_p67 = false;
			if (!X.enableFeatures_p67["enable_virtualdoll_line"].val) ok_p67 = false;
            if (!ok_p67)
            {
				if (dollline != null)
				{
					dollline.positionCount = 0;
				}
				return;
			}
			if (dollline != null && positionNeutral_p67!=null && sourceAtom_p67 != null)
			{
				dollline.positionCount = 2;
				dollline.SetPosition(0, positionNeutral_p67);
				dollline.SetPosition(1, sourceAtom_p67.mainController.transform.position);
			}

		}

		public void CheckReset_p67()
        {
			if (!GetTrackedController_p67()) return;
			var distance = GetDistance_p67();
			if (distance > X.sliders_p67["VDReset"].val)
			{
				SetTrackedController_p67();
			}
			

		}
		public void UpdatePosition_p67()
        {
			if (!GetTrackedController_p67()) return;
			if (X.choosers_p67["VDModeSticky"].val == "None") return;
			if (destinationRB == null) return;			
			if (sourceAtom_p67 == null) return;

			var t = destinationRB.transform;
			FreeControllerV3 raeController = sourceAtom_p67.GetStorableByID("control") as FreeControllerV3;
			if (raeController == null) return;




			sourceAtom_p67.mainController.transform.position = t.position;
			sourceAtom_p67.mainController.transform.rotation = t.rotation;

			sourceAtom_p67.mainController.currentPositionState = FreeControllerV3.PositionState.ParentLink;
			sourceAtom_p67.mainController.currentRotationState = FreeControllerV3.RotationState.ParentLink;
			sourceAtom_p67.mainController.SelectLinkToRigidbody(destinationRB, FreeControllerV3.SelectLinkState.PositionAndRotation);

		}
		void UpdateRB(string rbname)
		{
			destinationRB = SuperController.singleton.RigidbodyNameToRigidbody(rbname);
		}

		private Atom GetTrackedController_p67()
        {
			return SPQRUtils.GetAtomById_p67(X.AtomUidAddSuffix_p67("VirtualDollControl")); 
		}



		private void InitiateAliveVirtualDollControl()
		{
			sourceAtom_p67 = GetTrackedController_p67();
			sourceAtom_p67.mainController.currentRotationState = FreeControllerV3.RotationState.Off;
			sourceAtom_p67.mainController.transform.position = headController.transform.position + headController.transform.right * -0.5f;
			SPQRUtils.SetAtomStorableFloat_p67(sourceAtom_p67.uid, "scale", "scale", 0.500f);
			initiated["VirtualDollControl"] = true;

			SetTrackedController_p67();
			isStatic = true;
		}



		private void SetTrackedController_p67(string s = null)
        {
			if (!X.enableFeatures_p67["enable_virtualdoll"].val) return;
			sourceAtom_p67 = GetTrackedController_p67();

			if (sourceAtom_p67 == null) return;
			for(var i = 0; i < 5; i++)
            {
				trackedLog[i] = sourceAtom_p67.mainController.transform.position;
			}

			positionNeutral_p67 = trackedLog[4];
		}

		private void ProcessTrackedReset_p67()
		{
			var originalStatic = isStatic;
			if (sourceAtom_p67 == null) return;
			 isStatic = true;
			for (var i = 0; i < 4; i++)
			{
				trackedLog[i] = trackedLog[i + 1];
				if (i < 3 && trackedLog[i] != trackedLog[i + 2]) isStatic = false;
			}
			trackedLog[4] = sourceAtom_p67.mainController.transform.position;
			if (trackedLog[3] != trackedLog[4]) isStatic = false;

            if (isStatic && X.enableFeatures_p67["enable_autorecalibrate"].val)
			{
				positionNeutral_p67 = trackedLog[4];
			}
            if (!originalStatic)
            {
				processReset = true;
            }
		}

		private void ProcessTrackedPosition_p67()
        {
			if (sourceAtom_p67 == null) return;
			var distance = GetDistancePercent_p67();
			X.sliders_p67["VDPreview"].val = distance;

            switch (X.choosers_p67["VDMode"].val)
            {
				case "Ride":
					ProcessPositionRide_p67(distance);
					break;
				case "BJ":
					ProcessPositionBJ_p67(distance);
					break;
				case "Thrust":
					ProcessPositionThrust_p67(distance);
					break;
			}

			//SuperController.LogMessage("pos:" + sourceAtom_p67.mainController.transform.position.ToString());
			//SuperController.LogMessage("distance:" + GetDistancePercent_p67());
		}

		private void ProcessPositionRide_p67(float distance)
        {
			if (isStatic) return;
				//if (isStatic)
				//{
				//	if (processReset)
				//	{
				//		processReset = false;
				//	}
				//	else
				//	{
				//		return;
				//	}
				//}
				var neutral = chestController.transform.position +
				-0.4f * chestController.transform.up;
			var max = neutral - GetMaxMovement_p67() * hipController.transform.up;
			var pos = Vector3.Lerp(neutral, max, distance);
			hipController.transform.position = pos;
		}


		private void ProcessPositionBJ_p67(float distance)
		{
			if (isStatic) return;
			//if (isStatic)
			//{
			//	if (processReset)
			//	{
			//		processReset = false;
			//	}
			//	else
			//	{
			//		return;
			//	}
			//}
			var neutral = chestController.transform.position +
				0.2f * chestController.transform.up;
			var max = neutral + GetMaxMovement_p67() * headController.transform.forward;
			var pos = Vector3.Lerp(neutral, max, distance);
			headController.transform.position = pos;
		}

		private void ProcessPositionThrust_p67(float distance)
		{
			if (isStatic) return;
			//if (isStatic)
			//{
			//	if (processReset)
			//	{
			//		processReset = false;
			//	}
			//	else
			//	{
			//		return;
			//	}
			//}
			var neutral = chestController.transform.position +
				-0.4f * chestController.transform.up;
			var max = neutral + GetMaxMovement_p67() * hipController.transform.forward;
			var pos = Vector3.Lerp(neutral, max, distance);
			hipController.transform.position = pos;
		}


		public float GetDistance_p67()
        {
			if (positionNeutral_p67 == null) return 0f;
			if (sourceAtom_p67 == null) return 0f;
			var distance = Vector3.Distance(positionNeutral_p67, sourceAtom_p67.mainController.transform.position);
			return distance;
        }

		public float GetDistancePercent_p67()
        {
			var distance = GetDistance_p67();
			var max = GetMaxDistance_p67();
			if (distance > max) distance = max;

			return distance / max;
        }

		public float GetMaxDistance_p67()
        {
			var val =  X.sliders_p67["VDMaxDistance"].val;
			if (val <= 0.01f) val = 0.01f;
			return val;
		}
		public float GetMaxMovement_p67()
		{
			var val = X.sliders_p67["VDMaxMovement"].val;
			if (val <= 0.01f) val = 0.01f;
			return val;
		}

		public void callbackUpdateMode_p67(string s)
        {

        }

		public void callbackUpdateModeSticky_p67(string s)
		{
            switch (s)
            {
				case "None":
					UpdateRB(null);
                    sourceAtom_p67.ClearParentAtom();
                    sourceAtom_p67.mainController.currentPositionState = FreeControllerV3.PositionState.On;
                    sourceAtom_p67.mainController.currentRotationState = FreeControllerV3.RotationState.On;
					sourceAtom_p67.mainController.SelectLinkToRigidbody(null);

					break;
				case "VR Left Hand":
					UpdateRB("[CameraRig]:LeftHand");
					break;
				case "VR Right Hand":
					UpdateRB("[CameraRig]:RightHand");
					break;
				case "Tracker1":
				case "Tracker2":
				case "Tracker3":
				case "Tracker4":
				case "Tracker5":
				case "Tracker6":
				case "Tracker7":
				case "Tracker8":
					UpdateRB("[CameraRig]:"+ s);
					break;
				case "Mouse Scroll":
					UpdateRB("test:object");
					break;
			}

		}
		public void helpRecalibrate_p67()
        {

			SetTrackedController_p67();
		}
        public void ResetTracked_p67()
        {
			if (sourceAtom_p67 == null) return;
			for (var i = 0; i < 5; i++)
            {
				trackedLog[i] = sourceAtom_p67.mainController.transform.position;
			}
			
			positionNeutral_p67 = trackedLog[4];

        }
		public void helpSelectController_p67()
		{
			SetTrackedController_p67();
			SuperController.singleton.SelectController(sourceAtom_p67.mainController, false, false, false, true);

		}
	}
}